<?php
class utkCaptcha{
	protected $utkQuery;
	protected $utkSession;
	protected $utkCommon;
	
	// captcha setting
	protected $letters = "0123456789";
	protected $usedLetters = array();
	protected $lettersNum = 5;
	protected $letterPrecission = 15;
	protected $width = 160;
	protected $height = 50;
	protected $noise = 50;
	protected $lineNoise = 10;
	protected $padding = array(5,5,5,5);
	protected $fonts = array();
	protected $fontSize = array(20,22,24);
	protected $bg_color = array(255,255,255);
	protected $bg_color1 = array(128,128,128);
	protected $bg_color2 = array(200,200,200);
	protected $fgcolor = array(
							array(192,5,5),
							array(16,124,16),
							array(22,119,212),
							array(197,121,11)
						);
	
	function __construct(&$utkQuery,&$utkSession,&$utkCommon){
		$this->utkQuery = &$utkQuery;
		$this->utkSession = &$utkSession;
		$this->utkCommon = &$utkCommon;
    }
	
	public function getCaptchaImage(){
		$this->setFontList();
		$this->usedLetters = $this->getRandomLetters();
		$img = imagecreate($this->width,$this->height);
		imagecolorallocate($img,$this->bg_color[0],$this->bg_color[1],$this->bg_color[2]);
		
		$this->addNoise($img,$this->bg_color2);
		$this->printLetters($img);
		$this->addNoise($img,$this->bg_color1);
		
		header("Content-type: image/jpeg");
		imagejpeg($img);
		imagedestroy($img);
	}
	
	protected function setFontList(){
		$this->fonts = array();
		$font_dir = $this->utkCommon->getPathDataFont();
		$dh = dir($font_dir);
		while($entry = $dh->read()){
			if($entry != "." && $entry != ".."){
				$entry = explode(".",$entry);
				$ext = end($entry);
				if($ext == "ttf"){
					$this->fonts[] = $font_dir."/".$entry[0].".".$entry[1];
				}
			}
		}
	}
	
	protected function getRandomLetters(){
		$letters_max = (strlen($this->letters) - 1);
		
		for($i = 0;$i<$this->lettersNum;$i++){
			$letter_index = rand(0,$letters_max);
			$rtn_val[] = $this->letters[$letter_index];
		}
		return $rtn_val;
	}
	
	protected function addNoise(&$img,$color){
		$max_x = $this->width - 1;
		$max_y = $this->height - 1;
		$color = imagecolorallocate($img,$color[0],$color[1],$color[2]);
		
		for ($i=0;$i<$this->noise;$i++){
			if (rand(1,100) < $this->lineNoise) {
				imageline($img, rand(0,$max_x), rand(0,$max_y), rand(0,$max_x), rand(0,$max_y), $color);
			}else{
				imagesetpixel($img, rand(0,$max_x), rand(0,$max_y), $color);
			}
		}
	}
    
    protected function printLetters(&$img){
		global $_SESSION;
		$box_width = ($this->width - ($this->padding[1] + $this->padding[3])) / $this->lettersNum;
		$box_height = $this->height - ($this->padding[0] + $this->padding[2]); 
		
		for($i=0;$i<$this->lettersNum;$i++){
			$size_index = rand(0, count($this->fontSize) - 1);
			$size = $this->fontSize[$size_index];
			$angle = ((rand(0,($this->letterPrecission * 2)) - $this->letterPrecission) + 360) % 360;
			
			$x = $this->padding[3] + ($box_width * $i);
			$y = $this->padding[0] + $size + (($box_height - $size) / 2);
			
			$color_index = rand(0,count($this->fgcolor)-1);
			$color = $this->fgcolor[$color_index];
			$color = imagecolorallocate($img,$color[0],$color[1],$color[2]);
			
			$font_index = rand(0,count($this->fonts)-1);
			$font = $this->fonts[$font_index];
			
			imagettftext($img,$size,$angle,$x,$y,$color,$font,$this->usedLetters[$i]);
        }
		$_SESSION['img_reg'] = implode("",$this->usedLetters);
    }
}
?>
