<?php
include_once ('utkErrorMessage.inc');

class utkChecking {
	protected $utkQuery;
	protected $utkSession;
	protected $utkCommon;
	
	public function __construct(&$utkQuery, &$utkSession, &$utkCommon) {
		$this->utkQuery = &$utkQuery;
		$this->utkSession = &$utkSession;
		$this->utkCommon = &$utkCommon;
	}
	
	public function is_date($value) {
		$valid_day = array(0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		
		list ($date, $time) = split (" ", $value);
		list ($year, $month, $day) = split ("-", $date);
		
		if (((($year%4) == 0) && (($year%100) != 0)) || (($year % 400) == 0) ) {
			$valid_day[2] = 29;
		}
		
		$day = preg_replace("/^0/", "", $day);
		$month = preg_replace("/^0/", "", $month);
		
		if ($month < 1 || $month > 12) {
			return false;
		}
		else if ($day < 1 || $day > $valid_day[$month]) {
			return false;
		}
		
		if ($time != '') {
			list ($hour, $minute, $second) = split (":", $time);
			
			$hour = preg_replace("/^0/", "", $hour);
			$minute = preg_replace("/^0/", "", $minute);
			$second = preg_replace("/^0/", "", $second);
			
			if ($hour > 23) {
				return false;
			}
			else if ($minute > 59) {
				return false;
			}
			else if ($second > 59) {
				return false;
			}
		}
		
		return true;
	}
	
	public function checkAddress($field, &$value, $empty, $maxLength, $typ) {
		$value = trim($value);
		
		$message = '';
		
		if ($empty && strlen($value) == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(1, array($field));
		}
		else if (strlen($value) > $maxLength) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(2, array($field, $maxLength));
		}
		else if ($typ == 1 || $typ == 3) {
			$value = strtolower($value);
			if ($value != '') {
				$invalid = true;
				// Create the syntactical validation regular expression
				$regexp = "/^([_a-z0-9-]+)(\.[_a-z0-9-]+)*@([a-z0-9-]+)(\.[a-z0-9-]+)*(\.[a-z]{2,4})$/";

				// Validate the syntax
				if (preg_match($regexp, $value)) {
					if($typ == 1){
						list($username, $domaintld) = split("@", $value);
						// Validate the domain
						//if (checkdnsrr($domaintld, "ANY")) {
							$invalid = false;
						//}
					}elseif($typ == 3){
						$invalid = false;
					}
				}
				
				if ($invalid) {
					$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
					$message = $utkErrorMessage->getErrorMessage(0, array($field));
				}
			}
		}
		else if ($typ == 2) {			
			if ($value != '') {
				// Create the syntactical validation regular expression
				$regexp = "/^(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:.?+=&%@!\-\/]))?$/";
	
				// Validate the syntax
				if (!preg_match($regexp, $value)) {
					$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
					$message = $utkErrorMessage->getErrorMessage(0, array($field));
				}
			}
		}
		
		return $message;
	}
	
	public function checkPassword($field, &$value, $empty, $maxLength, $minLength) {
		$value = trim($value);
		
		$message = '';
		
		if ($empty && strlen($value) == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(1, array($field));
		}
		else if (strlen($value) < $minLength || strlen($value) > $maxLength) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(10, array($field, $minLength, $maxLength));
		}
		else {
			$regexp = "/^[a-zA-Z0-9]+$/";
			
			if (!preg_match($regexp, $value)) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				$message = $utkErrorMessage->getErrorMessage(10, array($field, $minLength, $maxLength));
			}
		}
		
		return $message;
	}
	
	//Input date must be '%Y-%m-%d or %Y-%m-%d %H:%i:%s'
	public function checkDate($field, &$value, $empty) {
		$value = trim($value);
		
		$message = '';
		
		if ($empty && strlen($value) == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(1, array($field));
		}
		else if (!$empty && strlen($value) == 0) {
			return $message;
		}
		else {
			if (!$this->is_date($value)) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				$message = $utkErrorMessage->getErrorMessage(0, array($field));
			}
		}
		
		return $message;
	}
	
	public function checkNumber($field, &$value, $empty, $maxLength, $minValue = null, $typ = 0) {
		$value = trim($value);
		
		$message = '';
		
		if ($typ == 1)
			$value = preg_replace("/^0+/", "", $value);
		
		if ($empty && strlen($value) == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(1, array($field));
		}
		else if (strlen($value) > $maxLength) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(2, array($field, $maxLength));
		}
		else if (!$empty && strlen($value) == 0) {
			return $message;
		}
		else if (!is_numeric($value)) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(3, array($field));
		}
		
		if (!is_null($minValue)) {
			if ($value < $minValue) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				$message = $utkErrorMessage->getErrorMessage(0, array($field));
			}
		}
		
		return $message;
	}
	
	public function checkString($field, &$value, $empty, $maxLength, $typ, $minValue = null) {
		$value = trim($value);
		$value = stripslashes($value);
		
		$message = '';
		
		if ($empty && strlen($value) == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(1, array($field));
		}
		else if (strlen($value) > $maxLength) {
			$value = substr($value, 0, $maxLength);
		}
		
		if ($typ == 1 && strlen($value) > 0) {
			$regexp = "/^[a-zA-Z ]+$/";

			if (!preg_match($regexp, $value)) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				$message = $utkErrorMessage->getErrorMessage(0, array($field));
			}
		}
		else if ($typ == 2 && strlen($value) > 0) {
			$regexp = "/^[a-zA-Z0-9 '\._\?!\-\&]+$/";

			if (!preg_match($regexp, $value)) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				$message = $utkErrorMessage->getErrorMessage(0, array($field));
			}
		}
		
		if (!is_null($minValue)) {
			if (strlen($value) < $minValue) {
				$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
				if ($minValue > 1)
					$minValue = "$minValue characters";
				else
					$minValue = "$minValue character";
				$message = $utkErrorMessage->getErrorMessage(7, array($field, $minValue));
			}
		}
		
		return $message;
	}
	
	//Input date must be '%Y-%m-%d or %Y-%m-%d %H:%i:%s'
	public function formatDate($input) {
		$mon	= array("", "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");
		
		list($date, $time) = split(" ", $input);
		
		list($year, $month, $day) = split("-", $date);
		$month = intval($month);
		
		$result = "$day " . $mon[$month] . " $year";
		
		if ($time != '')
			$result .= " $time"; 
		
		return $result;
	}
	
	public function checkSelected($field, &$value, $empty, $maxLength, $typ) {
		$message = '';
		$total_selected = 0;
		
		if (isset($value)) {
			if (is_array($value)) {
				$total_selected = count($value);
			}
			else {
				$total_selected = 1;
			}
			
			if ($typ == 1 || $typ == 2) {
				if (is_array($value)) {
					if (in_array("0-0", $value))
						$total_selected--;
				}
				else if ($value == "0-0")
					$total_selected--;
			}
			
			if ($typ == 2 || $typ == 3) {
				if (is_array($value)) {
					if (in_array("", $value))
						$total_selected--;
					if (in_array("NuLlS", $value))
						$total_selected--;
					else if (in_array("ReMaRkS", $value))
						$total_selected--;
					else if (in_array("line", $value))
						$total_selected--;
				}
				else if ($value == "NuLlS")
					$total_selected--;
				else if ($value == "ReMaRkS")
					$total_selected--;
				else if ($value == "line")
					$total_selected--;
			}
		}
		
		if ($empty && $total_selected == 0) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(4, array($field));
		}
		else if ($total_selected > $maxLength) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(5, array($maxLength, $field));
		}
		
		return $message;
	}
	
	public function checkRadio($field, &$value, $empty) {
		$message = '';
		
		if ($empty && !isset($value)) {
			$utkErrorMessage = new utkErrorMessage($this->utkQuery, $this->utkSession, $this->utkCommon);
			$message = $utkErrorMessage->getErrorMessage(6, array($field));
		}
		
		return $message;
	}
}
?>