<?php
class utkXMLParse {
	protected $directoryXML = '/home/httpd/xml/transaction/table';
	protected $utQuery = null;
	
	public function __construct(&$utQuery) {
		$this->utQuery = &$utQuery;
	}
	
	public function parseXML($filename, $error_log = null) {
		$fileXML = $this->directoryXML . "/$filename";
		$success = 0;
		$failed = 0;
		$message = '';
		
		$message .= "START PARSING XML\n$fileXML\n";
		
		if (file_exists($fileXML)) {
			if (!$doc = DOMDocument::load($fileXML)) {
				$message .= "Error loading $fileXML\nEND PARSING XML\n";
				return $message;
			}
		}
		else {
			$message .= "$fileXML is NOT FOUND\nEND PARSING XML\n";
			return $message;
		}
		
		$element_trans = $doc->getElementsByTagName("tra");
		$total_trans = $element_trans->length;
		
		for ($s = 0; $s < $total_trans; $s++) {
			$error = 0;
			$trans = $element_trans->item($s);
			$element_table = $trans->getElementsByTagName("tbl");
			$total_table = $element_table->length;
			
			for ($t = 0; $t < $total_table; $t++) {
				$table = $element_table->item($t);
					
				$table_name = $table->getAttribute("n");
				$trans_type = $table->getAttribute("p");
					
				if ($trans_type == "i") {
					$sql = "INSERT INTO $table_name(";
					$sql_val = "VALUES (";
					
					$element_field = $table->getElementsByTagName("fl");
					$total_field = $element_field->length;
					
					for ($f = 0; $f < $total_field; $f++) {
						$field = $element_field->item($f);
						
						$field_name = $field->getAttribute("n");
						$field_type = $field->getAttribute("t");
						$field_length = $field->getAttribute("l");
						$field_value = $field->nodeValue;
						
						$sql .= "$field_name,";
						
						if ($field_type == 't' && $field_value != '') {
							$sql_val .= "STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s'),";
						}
						else if ($field_type == 'h') {
							$sql_val .= "$field_value,";
						}
						else if ($field_type == 'o' && $field_value != '') {
							$lengthDecimal = $field_length - 1;
							$sql_val .= "CAST(? AS DECIMAL($lengthDecimal,0)),";
						}
						else {
							$sql_val .= "?,";
						}
					}	//for ($f = 0; $f < $total_field; $f++)
					
					$sql = substr($sql, 0, strlen($sql) - 1) . ") " . substr($sql_val, 0, strlen($sql_val) - 1) . ")";
					
//						echo "$sql<BR>";
					
					$stmt = $this->utQuery->parse($sql);
			
					for ($f = 0; $f < $total_field; $f++) {
						$field = $element_field->item($f);
						
						$field_name = $field->getAttribute("n");
						$field_type = $field->getAttribute("t");
						$field_length = $field->getAttribute("l");
						$field_value = $field->nodeValue;
						
						if ($field_type != 'h') {
							if ($field_type == 't' || $field_type == 'o')
								$field_type = 's';
							
							$this->utQuery->bindParam($stmt, $field_type, $field_value);
						}
					}	//for ($f = 0; $f < $total_field; $f++)
					
					if ($this->utQuery->execute($stmt)) {
						$result = $this->utQuery->rowAffected($stmt);
					}
					else {
						$failed++;
						$message .= "tra = $s FAILED. " . $this->utQuery->getError($stmt) . "\n";
					
						$this->utQuery->rollback();
						$this->utQuery->freeStatement($stmt);
						continue 2;
					}
					
					$this->utQuery->freeStatement($stmt);
				}	//if ($trans_type == "i")
				else if ($trans_type == "u") {
					$sql = "UPDATE $table_name SET ";
					
					$element_field = $table->getElementsByTagName("fl");
					$total_field = $element_field->length;
					
					for ($f = 0; $f < $total_field; $f++) {
						$field = $element_field->item($f);
						
						$field_name = $field->getAttribute("n");
						$field_type = $field->getAttribute("t");
						$field_length = $field->getAttribute("l");
						$field_value = $field->nodeValue;
						
						if ($field_type == 't' && $field_value != '') {
							$sql .= "$field_name=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s'),";
						}
						else if ($field_type == 'h') {
							$sql .= "$field_value,";
						}
						else if ($field_type == 'o' && $field_value != '') {
							$lengthDecimal = $field_length - 1;
							$sql_val .= "CAST(? AS DECIMAL($lengthDecimal,0)),";
						}
						else {
							$sql .= "$field_name=?,";
						}
					}	//for ($f = 0; $f < $total_field; $f++)
					
					$sql = substr($sql, 0, strlen($sql) - 1) . ")";
					
					$element_primary = $table->getElementsByTagName("ky");
					$total_primary = $element_primary->length;
					
					if ($total_primary > 0) {
						$sql .= " WHERE ";
						
						for ($k = 0; $k < $total_primary; $k++) {
							$primarykey = $element_primary->item($k);
							
							$field_name = $primarykey->getAttribute("n");
							$field_type = $primarykey->getAttribute("t");
							$field_length = $primarykey->getAttribute("l");
							$field_value = $primarykey->nodeValue;
							
							if ($field_type == 't' && $field_value != '') {
								$sql .= "$field_name=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s') AND ";
							}
							else if ($field_type == 'h') {
								$sql .= "$field_value AND ";
							}
							else if ($field_type == 'o' && $field_value != '') {
								$lengthDecimal = $field_length - 1;
								$sql .= "CAST(? AS DECIMAL($lengthDecimal,0)),";
							}
							else {
								$sql .= "$field_name=? AND ";
							}
						}	//for ($k = 0; $k < $total_primary; $k++)
						
						$sql = substr($sql, 0, strlen($sql) - 5);
					}	//if ($total_primary > 0)
					
//						echo "$sql<BR>";
					
					$stmt = $this->utQuery->parse($sql);
			
					for ($f = 0; $f < $total_field; $f++) {
						$field = $element_field->item($f);
						
						$field_name = $field->getAttribute("n");
						$field_type = $field->getAttribute("t");
						$field_length = $field->getAttribute("l");
						$field_value = $field->nodeValue;
						
						if ($field_type != 'h') {
							if ($field_type == 't' || $field_type == 'o')
								$field_type = 's';
								
							$this->utQuery->bindParam($stmt, $field_type, $field_value);
						}
					}	//for ($f = 0; $f < $total_field; $f++)
					
					for ($k = 0; $k < $total_primary; $k++) {
						$primarykey = $element_primary->item($k);
						
						$field_name = $primarykey->getAttribute("n");
						$field_type = $primarykey->getAttribute("t");
						$field_length = $primarykey->getAttribute("l");
						$field_value = $primarykey->nodeValue;
						
						if ($field_type != 'h') {
							if ($field_type == 't' || $field_type == 'o')
								$field_type = 's';
								
							$this->utQuery->bindParam($stmt, $field_type, $field_value);
						}
					}	//for ($k = 0; $k < $total_field; $k++)
					
					if ($this->utQuery->execute($stmt)) {
						$result = $this->utQuery->rowAffected($stmt);
					}
					else {
						$failed++;
						$message .= "tra = $s FAILED. " . $this->utQuery->getError($stmt) . "\n";
					
						$this->utQuery->rollback();
						$this->utQuery->freeStatement($stmt);
						continue 2;
					}
					
					$this->utQuery->freeStatement($stmt);
				}	//else if ($trans_type == "u")
				else if ($trans_type == "d") {
					$sql = "DELETE FROM $table_name";
					
					$element_field = $table->getElementsByTagName("fl");
					$total_field = $element_field->length;
					
					$element_primary = $table->getElementsByTagName("ky");
					$total_primary = $element_primary->length;
					
					if ($total_primary > 0) {
						$sql .= " WHERE ";
						
						for ($k = 0; $k < $total_primary; $k++) {
							$primarykey = $element_primary->item($k);
							
							$field_name = $primarykey->getAttribute("n");
							$field_type = $primarykey->getAttribute("t");
							$field_length = $primarykey->getAttribute("l");
							$field_value = $primarykey->nodeValue;
							
							if ($field_type == 't' && $field_value != '') {
								$sql .= "$field_name=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s') AND ";
							}
							else if ($field_type == 'h') {
								$sql .= "$field_name=$field_value AND ";
							}
							else if ($field_type == 'o' && $field_value != '') {
								$lengthDecimal = $field_length - 1;
								$sql .= "CAST(? AS DECIMAL($lengthDecimal,0)),";
							}
							else {
								$sql .= "$field_name=:$field_name AND ";
							}
						}	//for ($k = 0; $k < $total_primary; $k++)
						
						$sql = substr($sql, 0, length($sql) - 5);
					}	//if ($total_primary > 0)
					
//						echo "$sql<BR>";
					
					$stmt = $this->utQuery->parse($sql);
			
					for ($k = 0; $k < $total_primary; $k++) {
						$primarykey = $element_primary->item($k);
						
						$field_name = $primarykey->getAttribute("n");
						$field_type = $primarykey->getAttribute("t");
						$field_length = $primarykey->getAttribute("l");
						$field_value = $primarykey->nodeValue;
						
						if ($field_type != 'h') {
							if ($field_type == 't' || $field_type == 'o')
								$field_type = 's';
								
							$this->utQuery->bindParam($stmt, $field_type, $field_value);
						}
					}	//for ($k = 0; $k < $total_field; $k++)
					
					if ($this->utQuery->execute($stmt)) {
						$result = $this->utQuery->rowAffected($stmt);
					}
					else {
						$failed++;
						$message .= "tra = $s FAILED. " . $this->utQuery->getError($stmt) . "\n";
					
						$this->utQuery->rollback();
						$this->utQuery->freeStatement($stmt);
						continue 2;
					}
					
					$this->utQuery->freeStatement($stmt);
				}	//else if ($trans_type == "d")
				else {
					$this->utQuery->rollback();
					$failed++;
					$message .= "tra = $s FAILED. TYPE = $trans_type\n";
					continue 2;
				}
			}	//for ($t = 0; $t < $total_table; $t++)
		
			if ($error == 0) {
				$element_file = $trans->getElementsByTagName("fil");
				$total_file = $element_file->length;
				
				for ($i = 0; $i < $total_file; $i++) {
					$file = $element_file->item($i);
			
					$file_name = $file->getAttribute("n");
					$file_type = $file->getAttribute("t");
					$file_value = $file->nodeValue;
					
					if ($trans_type == "d") {
						system("rm -f $file_name");
					}
					else {
						if ($file_type == "e") {
							$handle = fopen($file_name, "wb");
							fwrite($handle, base64_decode($file_value));
							fclose($handle);
						}
					}
				}	//for ($i = 0; $i < $total_file; $i++)
			}	//if ($error == 0)
				
			if ($error == 0) {
				$element_call = $trans->getElementsByTagName("cal");
				$total_call = $element_call->length;
	
				for ($c = 0; $c < $total_call; $c++) {
					$methodcall = $element_call->item($c);
					
					$file_call = $methodcall->getAttribute("n");
			
					$element_param = $methodcall->getElementsByTagName("pr");
					$total_param = $element_param->length;
					
					for ($p = 0; $p < $total_param; $p++) {
						$methodparam = $element_param->item($p);
						
						$param_argv = $methodparam->getAttribute("n");
						$param_val = $methodparam->nodeValue;
						
						$file_call .= " $param_argv $param_val";
					}	//for ($p = 0; $p < $total_param; $p++)
					
					system("perl $file_call");
				}	//for ($c = 0; $c < $total_call; $c++)
			}	//if ($error == 0)
			
			if ($error == 0) {
				$this->utQuery->commit();
				$success++;
			}
			else {
				$failed++;
			}
		}	//for ($s = 0; $s < $total_trans; $s++)
		
		$message .= "SUCCESS = $success\n";
		$message .= "FAILED = $failed\n";
		$message .= "END PARSING XML\n";
		
		return $message;
	}
}
?>
