<?php
include_once('utkMyStmt.inc');

class utkQuery {
	protected $utkDbase = null;
	protected $utkSession = null;
	protected $server = '';
	protected $conn = null;
	
	public function __construct(&$utkDbase, &$utkSession, $server = 'master', $connect = true) {
		$this->utkDbase = &$utkDbase;
		$this->utkSession = &$utkSession;
		$this->server = strtolower($server);
		
		if ($connect)
			$this->conn = $this->utkDbase->connect($this->utkSession, $this->server);
	}
	
	function query($sql) {
		return $this->conn->query($sql);
	}
	
	public function is_connect() {
		if ($this->conn) {
			if ($this->conn->ping())
				return true;
			else
				return false;
		}
		else
			return false;
	}
	
	public function &getSession() {
		return $this->utkSession;
	}
	
	public function reconnect() {
		$this->utkDbase->disconnect($this->server);
		$this->conn = $this->utkDbase->connect($this->utkSession, $this->server);
	}
	
	public function &parse($sql) {
		global $_SERVER;
		//set timezone
		//$tz = "SET time_zone='Asia/Jakarta'";
		$tz = "SET time_zone='+07:00'";
		$stmt_tz = new utkMyStmt($this->conn, $tz);
		$this->execute($stmt_tz);
		$this->freeStatement($stmt_tz);
		
		$stmt = new utkMyStmt($this->conn, $sql);
		if ($stmt->errno != 0) {
			if ($this->utkDbase->isShowError())
				echo "PARSE ERROR: " . $this->getError($stmt) . "<br>";
			else
				error_log ("PARSE ERROR: " . $this->getError($stmt) . " in " . $_SERVER['SCRIPT_FILENAME'], 0);
		}

		return $stmt;
	}
	
	public function bindParam(&$stmt, $type, $value) {
		$stmt->setParam($type, $value);
	}
	
	public function execute(&$stmt) {
		global $_SERVER;
				
		$stmt->bindParam();
		if ($stmt->execute()) {
			$stmt->bindResult();
			return true;
		}
		else {
			if ($this->utkDbase->isShowError())
				echo "EXECUTE ERROR: " . $this->getError($stmt) . "<br>";
			else
				error_log ("EXECUTE ERROR: " . $this->getError($stmt) . " in " . $_SERVER['SCRIPT_FILENAME'], 0);
			
			return false;
		}
	}
	
	public function fetch(&$stmt, $clone = false) {
		if ($stmt->fetch()) {
			$stmt->addNumRow();
			return $stmt->getResult($clone);
		}
		else {				
			return null;
		}
	}
	
	public function &fetchAll(&$stmt) {
		$result = array();
		$count = 0;
		
		while ($stmt->fetch()) {
			$stmt->addNumRow();
			$result[$count++] = $stmt->getResult(true);
		}
		
		$stmt->close();
		
		if (count($result) == 0) {				
			$result = null;
		}
		return $result;
	}
	
	// For SELECT syntax
	public function rowCount(&$stmt) {
		return $stmt->getNumRow();
	}
	
	// For INSERT / UPDATE / DELETE syntax
	public function rowAffected(&$stmt) {
		return $stmt->affected_rows;
	}
	
	public function getError(&$stmt) {
		return $stmt->errno . " " . $stmt->error;
	}
	
	public function commit(&$utkSessionLastUpdate = '') {
		$this->conn->commit();
		
		if (!empty($utkSessionLastUpdate))
			$utkSessionLastUpdate->setLastUpdate();
	}
	
	public function rollback() {
		$this->conn->rollback();
	}
	
	public function freeStatement(&$stmt) {
		if ($stmt->is_open())
			$stmt->close();
	}
}
?>
