<?php
abstract class utkTable {
	abstract protected function getTableName();
	abstract protected function getFieldName();
	abstract protected function getUpdateKeyXMLTable();
	abstract protected function getKeyXMLTable();
	abstract protected function setTransaction($type);
	
	public function __construct() {
	}
	
	public function getMetaData(&$utkQuery) {
		$fieldName = array();
		
		$sql = "SELECT *  FROM ". $this->getTableName() . " LIMIT 1";
		$stmt = $utkQuery->parse($sql);

		$fieldName = array();
		$resultMetaData = $stmt->result_metadata();

		if ($resultMetaData) {
			while ($field = $resultMetaData->fetch_field()) {
				$fieldName[strtoupper($field->name)] = array ($this->getType($field->type, $field->length), null);
			}
			
			$resultMetaData->free();
		}
		
		$utkQuery->freeStatement($stmt);
		
		return $fieldName;
	}
	
	public function getType($type, $length) {
		if ($type == 1 || $type == 2 || $type == 3 || $type == 9) 
			return 'i';
		else if ($type == 8)
			return 'd';
		else if ($type == 246) {
			if ($length > 9 && $length < 19)
				return 'd';
			else if ($length >= 19)
				return 'o';
			else
				return 'i';
		}
		else if ($type == 253)
			return 's';
		else if ($type == 12)
			return 't';
	}
	
	public function getUpdateXMLTable() {
		$result = array();
		
		$fieldName = $this->getFieldName();
		$primarykey = $this->getKeyXMLTable();
		
		while(list ($key, $value) = each($fieldName)) {
			if (!is_null($value[1]) && !array_key_exists($key, $primarykey))
				$result[$key] = array($value[0], $value[1], $value[2]);
		}

		return $result;
	}
	
	public function insert(&$utkQuery) {
		$result = -1;
		$this->setTransaction('i');
		$table = $this->getTableName();
		$field = $this->getFieldName();
		
		if (count($field) > 0) {
			$sql = "INSERT INTO $table (";
			$sql_val = " VALUES(";
			
			while (list($key, $value) = each($field)) {
				$sql .= "$key,";

				if ($value[0] == 't' && $value[1] != '') {
					$sql_val .= "STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s'),";
				}
				else if ($value[0] == 'h') {
					$sql_val .= "$value[1],";
				}
				else if ($value[0] == 'o' && $value[1] != '') {
					$lengthDecimal = $value[2] - 1;
					$sql_val .= "CAST(? AS DECIMAL($lengthDecimal,0)),";
				}
				else {
					$sql_val .= "?,";
				}
			}	//while (list($key, $value) = each($field))
			
			$sql = substr($sql, 0, strlen($sql) - 1) . ") " . substr($sql_val, 0, strlen($sql_val) - 1) . ")";
			//echo ($sql);
			$stmt = $utkQuery->parse($sql);
			
			reset($field);
			while (list($key, $value) = each($field)) {
				if ($value[0] != 'h') {
					if ($value[0] == 't' || $value[0] == 'o')
						$value[0] = 's';
					
					$utkQuery->bindParam($stmt, $value[0], $value[1]);
				}
			}	//while (list($key, $value) = each($field))
			
			if ($utkQuery->execute($stmt)) {
				$result = $utkQuery->rowAffected($stmt);
			}
			
			$utkQuery->freeStatement($stmt);
		}	//if (count($field) > 0)
		
		return $result;
	}
	
	public function update(&$utkQuery, $key = false) {
		$result = -1;
		$this->setTransaction('u');
		$table = $this->getTableName();
		
		if ($key) {
			$field = $this->getUpdateKeyXMLTable();
		}
		else {
			$field = $this->getUpdateXMLTable();
		}
		
		$primarykey = $this->getKeyXMLTable();
		//var_dump($field);
		if (count($field) > 0 && count($primarykey) > 0) {
			$sql = "UPDATE $table SET ";
			
			while (list($key, $value) = each($field)) {
				if ($value[0] == 't' && $value[1] != '') {
					$sql .= "$key=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s'),";
				}
				else if ($value[0] == 'h') {
					$sql .= "$key=$value[1],";
				}
				else if ($value[0] == 'o' && $value[1] != '') {
					$lengthDecimal = $value[2] - 1;
					$sql .= "$key=CAST(? AS DECIMAL($lengthDecimal,0)),";
				}
				else {
					$sql .= "$key=?,";
				}
			}	//while (list($key, $value) = each($field))
			
			$sql = substr($sql, 0, strlen($sql) - 1) . " WHERE ";
			
			while (list($key, $value) = each($primarykey)) {
				if ($value[0] == 't' && $value[1] != '') {
					$sql .= "$key=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s') AND ";
				}
				else if ($value[0] == 'h') {
					$sql .= "$key=$value[1] AND ";
				}
				else if ($value[0] == 'o' && $value[1] != '') {
					$lengthDecimal = $value[2] - 1;
					$sql .= "$key=CAST(? AS DECIMAL($lengthDecimal,0)) AND ";
				}
				else {
					$sql .= "$key=? AND ";
				}
			}	//while (list($key, $value) = each($primarykey))
			
			$sql = substr($sql, 0, strlen($sql) - 5);
			
			//echo "\n\n\n$sql\n\n\n";
			
			$stmt = $utkQuery->parse($sql);
			
			reset($field);
			while (list($key, $value) = each($field)) {
				if ($value[0] != 'h') {
					if ($value[0] == 't' || $value[0] == 'o')
						$value[0] = 's';
						
					$utkQuery->bindParam($stmt, $value[0], $value[1]);
					//echo "\n---- typ=".$value[0]." val=".$value[1];
				}
			}	//while (list($key, $value) = each($field))
			
			reset($primarykey);
			while (list($key, $value) = each($primarykey)) {
				if ($value[0] != 'h') {
					if ($value[0] == 't' || $value[0] == 'o')
						$value[0] = 's';

					$utkQuery->bindParam($stmt, $value[0], $value[1]);
					//echo "\n---- typ=".$value[0]." val=".$value[1];
				}
			}	//while (list($key, $value) = each($primarykey))
			
			if ($utkQuery->execute($stmt)) {
				$result = $utkQuery->rowAffected($stmt);
			}
			
			$utkQuery->freeStatement($stmt);
		}	//if (count($field) > 0 && count($primarykey) > 0)
		
		return $result;
	}
	
	public function delete(&$utkQuery) {
		$result = -1;
		$this->setTransaction('d');
		$table = $this->getTableName();
		$primarykey = $this->getKeyXMLTable();
		
		if (count($primarykey) > 0) {
			$sql = "DELETE FROM $table WHERE ";
			
			while (list($key, $value) = each($primarykey)) {
				if ($value[0] == 't' && $value[1] != '') {
					$sql .= "$key=STR_TO_DATE(?, '%Y-%m-%d %H:%i:%s') AND ";
				}
				else if ($value[0] == 'h') {
					$sql .= "$key=$value[1] AND ";
				}
				else if ($value[0] == 'o' && $value[1] != '') {
					$lengthDecimal = $value[2] - 1;
					$sql .= "$key=CAST(? AS DECIMAL($lengthDecimal,0)) AND ";
				}
				else {
					$sql .= "$key=? AND ";
				}
			}	//while (list($key, $value) = each($primarykey))
			
			$sql = substr($sql, 0, strlen($sql) - 5);
			
			//echo "$sql<BR>";
			
			$stmt = $utkQuery->parse($sql);
			
			reset($primarykey);
			while (list($key, $value) = each($primarykey)) {
				if ($value[0] != 'h') {
					if ($value[0] == 't' || $value[0] == 'o')
						$value[0] = 's';
						
					$utkQuery->bindParam($stmt, $value[0], $value[1]);
					//print_r($value);
				}
			}	//while (list($key, $value) = each($primarykey))
			
			if ($utkQuery->execute($stmt)) {
				$result = $utkQuery->rowAffected($stmt);
			}
			
			$utkQuery->freeStatement($stmt);
		}	//if (count($primarykey) > 0)
		
		return $result;
	}
}
?>
